/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
/*
 * general.cc
 * Copyright (C) 2016 Unknown <francois@gautrais.eu>
 *
 * histodex is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * histodex is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "general.h"

#include <cassert>

General::General(Json::Value& v) : Data(v)
{
	m_data_type=DATA_TYPE_GENERAL;
	if(v.size()) set(v);
	else
	{
		m_name="";
		m_nStages=3;
		m_dir=".";
	}
}

General::General() : Data()
{
	m_data_type=DATA_TYPE_GENERAL;
	m_nStages=1;
}

General::~General()
{
}

void General::set(Json::Value& v)
{
	m_name = v["name"].asString();
	m_nStages = v["stages"].asInt();
	m_dir = v["dir"].asString();
	m_map = v["map"].asString();
}

const Data& General::operator=(const General& a)
{
	m_name=a.m_name;
	m_dir=a.m_dir;
	m_nStages=a.m_nStages;
	m_map=a.m_map;
	return *this;
}

Json::Value General::getJson(bool genFinal)
{
	Json::Value v(Json::objectValue);
	v["name"] = m_name;
	v["stages"] = m_nStages;
	v["map"] = m_map;
	if(!genFinal) v["dir"] = m_dir;
	return v;
}

General& General::cast(Data& d)
{
	if(d.getDataType ()!=DATA_TYPE_GENERAL)
		assert(0);
	return (General&)d;
}



const std::string& General::getMap()
{
	return m_map;
}