/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
/*
 * generator.h
 * Copyright (C) 2016 Fran??ois Gautrais <francois@gautrais.eu>
 *
 * histodex is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * histodex is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _GENERATOR_H_
#define _GENERATOR_H_


#include <vector>
#include <json/json.h>
#include "general.h"

#define LOG_WARNING 0
#define LOG_ERROR 1
#define LOG_CRITICAL 2

class GWindow;

class Area;
class Resource;
class Stage;
class Generator
{
public:
	Generator(General& g, std::vector<Area>& a,std::vector<Resource>& r,
	          std::vector<Stage>& s, GWindow* ui);
	virtual ~Generator();
	std::string getLog();
	void generate();
	void log(const std::string& l);
	int  getErrorsCount();
	int  getCriticalErrorsCount();
	int  getWarningCount();

protected:
	General				   m_gen;
	std::vector<Resource>  m_resources;
	std::vector<Area>	   m_areas;
	std::vector<Stage>	   m_stages;
	std::vector<std::string>	   m_log;
	std::vector<std::string>	   m_errors;
	int					   m_nCriticalErrors;
	int					   m_nErrors;
	int					   m_nWarning;
	std::string			   m_pathRaw;
	std::string			   m_pathDrawable;
	GWindow*			m_ui;

protected: 
	void prepare();
	void error(const std::string& l, int level=LOG_WARNING);
	void generateMap();
	void generateResource(Resource& r);
	void generateResource3D(Resource& r);
	void generateResourceImage(Resource& r);
	void generateResourceAudio(Resource& r);
	void generateResourceVideo(Resource& r);
	 
	void check();
	void checkAreas();
	void checkStages();
	void checkResources();
	Area* findArea(const std::string& name);
	Resource* findResource(const std::string& name);
	Stage* findStage(int i);

	std::string newFilename(const std::string& file); 
	void		copyObj(const std::string& obj, const std::string& dir, const std::string& name);
	void		copyMtl(const std::string& mtl, const std::string& dir);
	void		copyImage(const std::string& mtl, const std::string& dir);
	
private:

};

#endif // _GENERATOR_H_

