/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
/*
 * merkaartot.cc
 * Copyright (C) 2016 Unknown <francois@gautrais.eu>
 *
 * histodex is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * histodex is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "merkator.h"
#include "area.h"

Merkaartot::Merkaartot(const std::string& file)
{
	
	TiXmlDocument doc(file);
	bool loadOkay = doc.LoadFile();
	if (loadOkay)
	{
		printf("\n%s:\n", file.c_str());
		load( &doc); // defined later in the tutorial
		resolve();
	}
	else
	{
		printf("Failed to load file \"%s\"\n", file.c_str());
	}
}

Merkaartot::~Merkaartot()
{
}

Node* Merkaartot::findNode(const std::string& id)
{
	for(int i=0; i<m_nodes.size(); i++)
		if(m_nodes[i].m_id == id)
			return &m_nodes[i];
	return NULL;
}

Way* Merkaartot::findWay(const std::string& nom)
{
	for(int i=0; i<m_ways.size(); i++)
	{
		Way& n = m_ways[i];
		for(int j=0; j<n.m_keys.size(); j++)
		{
			if(n.m_keys[j]=="name" && n.m_values[j]==nom)
				return &n;
		}
	}
	return NULL;
}


void Merkaartot::load( TiXmlNode* pParent)
{
	if ( !pParent );

	TiXmlNode* pChild;
	TiXmlText* pText;
	int t = pParent->Type();
	int num;

	switch ( t )
	{
	case TiXmlNode::TINYXML_ELEMENT:
		if(!strcmp(pParent->Value(), "way" ))
		{
			
			printf("W OK %s\n", pParent->ToElement ()->Attribute("id"));
			m_ways.push_back(Way(pParent->ToElement ()));
		}
	    if(!strcmp(pParent->Value(), "node" ))
		{
			
			printf("N OK %s\n", pParent->ToElement ()->Attribute("id"));
			m_nodes.push_back(Node(pParent->ToElement ()));
		}
		
		break;

	}
	for ( pChild = pParent->FirstChild(); pChild != 0; pChild = pChild->NextSibling()) 
	{
		load( pChild);
	}
	
}


void Merkaartot::resolve()
{
	for(int i=0; i<m_ways.size(); i++)
	{
		Way& w = m_ways[i];
		for(int j=0; j<w.m_refs.size(); j++)
			w.m_nodes.push_back (findNode (w.m_refs[j]));
	}

	for(int i=0; i<m_nodes.size(); i++)
	{
		Node& n = m_nodes[i];
		for(int j=0; j<n.m_keys.size(); j++)
		{
			if(n.m_keys[j]=="parent")
			{
				Way* w = findWay (n.m_values[j]);
				if(w)
				{
					w->m_points.push_back (&n);
				}
			}
		}
	}
}

Json::Value Merkaartot::getJson()
{
	Json::Value a(Json::arrayValue);
	
	for(int i=0; i<m_ways.size(); i++)
	{
		Area w(m_ways[i]);
		a[i]=w.getJson ();
	}

	return a;
}
