/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
/*
 * merkaartot.h
 * Copyright (C) 2016 Unknown <francois@gautrais.eu>
 *
 * histodex is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * histodex is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _MERKAARTOT_H_
#define _MERKAARTOT_H_
int _main(int argc, char* argv[]);

#include <tinyxml.h>
#include <iostream>
#include <cstdlib>
#include <vector>
#include <json/json.h>

class Node
{
	 public:
		virtual ~Node(){}
		 
		Node(const TiXmlElement* node)
		{
			const TiXmlElement* curr = node->FirstChildElement ();
			m_id=node->Attribute("id");
			m_lon=atof(node->Attribute("lon"));
			m_lat=atof(node->Attribute("lat"));
			while(curr)
			{
				std::string type = curr->Value();
				if(type == "tag")
				{
					m_keys.push_back(curr->Attribute("k"));
					m_values.push_back(curr->Attribute("v"));
				}
				curr=curr->NextSiblingElement();
			}
		}

		void print()
		{
			//std::cout << "\t\tNoeud : "<< m_id << " ("<< m_lon <<", " <<  m_lat <<")" << std::endl;
			for(int i=0; i<m_keys.size(); i++)
				;//std::cout << "\t\t\t'"<< m_keys[i] << "' = '" << m_values[i] << "'" << std::endl;
		}
		
		std::vector<std::string> m_keys;
		std::vector<std::string> m_values;
		double					 m_lon;
		double					 m_lat;
		std::string				 m_id;
};


class Way
{
	public:
		virtual ~Way(){}
		Way(const TiXmlElement* node)
		{
			const TiXmlElement* curr = node->FirstChildElement ();
			m_id=node->Attribute("id");
			while(curr)
			{
				std::string type = curr->Value();
				if(type == "tag")
				{
					m_keys.push_back(curr->Attribute("k"));
					m_values.push_back(curr->Attribute("v"));
				}
				if(type == "nd")
				{
					m_refs.push_back(curr->Attribute("ref"));
				}
				curr=curr->NextSiblingElement();
			}
		}

		void print()
		{
			std::cout << "Way : "<< m_id << std::endl;
			std::cout << "\tShape: : "<< m_id << std::endl;
			for(int i=0; i<m_nodes.size(); i++)
				m_nodes[i]->print();
			
			std::cout << "\tTags: : "<< m_id << std::endl;
			for(int i=0; i<m_keys.size(); i++)
				std::cout << "\t\t'"<< m_keys[i] << "' = '" << m_values[i] << std::endl;
			std::cout <<  std::endl;
			
			std::cout << "\tPoints: : "<< m_id << std::endl;
			for(int i=0; i<m_points.size(); i++)
				m_points[i]->print();
		}
		
		std::vector<std::string> m_keys;
		std::vector<std::string> m_values;
		std::vector<Node*>		 m_nodes;
		std::vector<Node*>		 m_points;
		std::vector<std::string> m_refs;
		std::string				 m_id;
};


class Merkaartot
{
	public:
		Merkaartot(const std::string& file);
		virtual ~Merkaartot();
		Json::Value getJson();
	protected:
		std::vector<Node> m_nodes;
		std::vector<Way> m_ways;
	protected:
		void load( TiXmlNode* pParent);
		void resolve();
		Node* findNode(const std::string& id);
		Way*  findWay(const std::string& name);
	private:
	

};

#endif // _MERKAARTOT_H_

