/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
/*
 * os.cc
 * Copyright (C) 2016 Fran??ois Gautrais <francois@gautrais.eu>
 *
 * histodex is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * histodex is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "os.h"
#include <iostream>
#include <fstream>
#if defined(__linux__)
#include <unistd.h>

#include <sys/time.h>
#include <sys/vfs.h>
#include <sys/statvfs.h>
#include <sys/stat.h>
#include <sys/types.h>
#else
#include <direct.h>
#endif
OS::Path::Path(const std::string& file)
{
	int slash = file.find_last_of("/");
	int dot = file.find_last_of(".");
	#if defined(__linux__)
	m_pwd=OS::pwd();
	if(file[0]=='/')
	{
		m_dir=file.substr(0, slash+1);
	}else
	{
		if(slash!=std::string::npos) m_dir=file.substr(0, slash+1);
		else m_dir="";
		
	}
	if(dot== std::string::npos)
	{
		m_ext="";
		m_name=file.substr(slash+1);
	}
	else
	{
		m_name=file.substr(slash+1, dot-slash-1);
		m_ext=file.substr(dot+1);
	}
	   
	#else

	#endif
}

void OS::Path::print()
{
	//std::cout << m_dir << ", " << m_name << ", " << m_ext << std::endl;
}

std::string OS::Path::getFileName()
{
	if(m_ext=="")
	{
		if(m_name=="")
			return m_dir;
		return m_dir+m_name;
	}
	return m_dir+m_name+"."+m_ext;
}

void OS::mkdir(const std::string& dir)
{
	#if defined(__linux__)
	::mkdir(dir.c_str(), 0755);
	#else
	::mkdir(dir.c_str());
	
	#endif
}


std::string OS::pwd()
{
	char tmp[1024];
	#if defined(__linux__)
	getcwd(tmp, 1023);
	return std::string(tmp)+"/";
	#else
	getcwd(tmp, 1023);
	return std::string(tmp)+"\\";
	
	#endif
}


void OS::copy(const std::string& src, const std::string& dst)
{
	
	std::ifstream  asrc(src, std::ios::binary);
    std::ofstream  adst(dst,   std::ios::binary);

    adst << asrc.rdbuf();
}




std::string OS::concatPath(const std::string& src, const std::string& dst)
{
	#if defined(__linux__)
		if(src.size()>=0  && src.find_last_of("/") == src[src.size()-1] )
			return src+dst;
		else
			return src+"/"+dst;
	#else
		if(src.size()>=0  && src.find_last_of("\\") == src[src.size()-1] )
			return src+dst;
		else
			return src+"\\"+dst;
	#endif
}
