/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
/*
 * stage.cc
 * Copyright (C) 2016 Fran??ois Gautrais <francois@gautrais.eu>
 *
 * histodex is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * histodex is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "stage.h"
#include <cassert>

Stage::Stage(Json::Value v) : Data(v)
{
	m_data_type=DATA_TYPE_STAGE;
	if(v.size()) set(v);
	else
	{
		m_nRes=1;
	}
}


Stage::Stage() : Data()
{
	m_data_type=DATA_TYPE_STAGE;
	m_nRes=1;
}

Stage::~Stage()
{
}

void Stage::set(Json::Value& v)
{
	m_stage=v["stage"].asInt();
	m_resource=v["resource"].asString();
	m_nRes=v["n_resource_step"].asInt();
	m_transition=v["transition"].asBool();
}
 
const std::string& Stage::getResource() { return m_resource; }
void Stage::setResource(const std::string& r) { m_resource=r; }
int Stage::getStage() { return m_stage; }
void Stage::setStage(int stage) { m_stage=stage; }
int Stage::getNRes() { return m_nRes; }
void Stage::setNRes(int res) { m_nRes = res; }

Json::Value Stage::getJson(bool b)
{
	Json::Value v(Json::objectValue);
	v["resource"]=m_resource;
	v["stage"]=m_stage;
	v["n_resource_step"]=m_nRes;
	v["transition"]=m_transition;
	return v;
}


Stage& Stage::cast(Data& d)
{
	if(d.getDataType ()!=DATA_TYPE_STAGE)
		assert(0);
	return (Stage&)d;
}


void	Stage::setTransition(bool b)
{
	m_transition=b;
}

bool	Stage::getTransition()
{
	return m_transition;
}