/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
/*
 * tab.cc
 * Copyright (C) 2017 Fanch <francois@gautrais.eu>
 *
 * histodexc is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * histodexc is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "tab.h"
#include <cstring>
std::string gtk_tree_view_get_str_selected(GtkTreeView* gt, bool* valid)
{
	GtkTreeIter iter;
	GtkTreeModel *model;
	gchar *value;
	GtkTreeSelection* sel;
	std::string ret;

	if(valid) *valid=false;
	sel=gtk_tree_view_get_selection (gt);
	if (gtk_tree_selection_get_selected(
		GTK_TREE_SELECTION(sel), &model, &iter)) {

		gtk_tree_model_get(model, &iter, 0, &value,  -1);
		ret=value;
		g_free(value);
		if(valid) *valid=true;
	}
	return ret;
}

bool gtk_tree_view_set_str_selected(GtkTreeView* gt, const std::string& str)
{
	GtkTreeIter iter;
	GtkTreeModel *model;
	gchar *value;
	GtkTreeSelection* sel;
	std::string ret;

	sel=gtk_tree_view_get_selection (gt);
	if (gtk_tree_selection_get_selected(
			GTK_TREE_SELECTION(sel), &model, &iter)) {
		gtk_list_store_set(GTK_LIST_STORE(model), 
		                   &iter, 0, str.c_str(), -1);
		return true;
	}
	return false;
}

char* gtk_text_view_get_text(GtkTextView* tv)
{
	GtkTextBuffer* buffer = gtk_text_view_get_buffer (tv);
	GtkTextIter start_find, end_find;
	gtk_text_buffer_get_start_iter(buffer, &start_find);
	gtk_text_buffer_get_end_iter(buffer, &end_find);
	return gtk_text_buffer_get_text (buffer,
	                                 &start_find,
	                                 &end_find,
	                                 TRUE);
}

void gtk_text_view_set_text(GtkTextView* tv, const char* s)
{
	GtkTextBuffer* buffer = gtk_text_view_get_buffer (tv);
	GtkTextIter start_find, end_find;
	gtk_text_buffer_get_start_iter(buffer, &start_find);
	gtk_text_buffer_get_end_iter(buffer, &end_find);
	gtk_text_buffer_delete(buffer, &start_find, &end_find);
	gtk_text_buffer_get_start_iter(buffer, &start_find);
	gtk_text_buffer_insert(buffer, &start_find, s, strlen(s));
}

void gtk_tree_model_get_iter_last(GtkTreeModel* tm, GtkTreeIter* iter)
{
	GtkTreeIter *old=iter;
	gtk_tree_model_get_iter_first (tm, iter);
	while(gtk_tree_model_iter_next (tm, iter))
		old=iter;
	*iter=*old;
}


Tab::Tab(GtkWidget *window, GtkBuilder *builder)
{
	m_window=window;
	m_builder=builder;
}

Tab::~Tab()
{
}

void Tab::clearModified()
{
	m_isModified=false;
}

void Tab::setModified()
{
	m_isModified=true;
}

bool Tab::getModified()
{
	return m_isModified;
}

 
void		Tab::setupTab(Data& a)
{
	m_data=&a;
}

const Tab&		Tab::operator=(const Tab& c)
{
	m_window=c.m_window;
	m_builder=c.m_builder;
	m_data=c.m_data;
	return *this;
}
void Tab::dialog_error(const char* str)
{
	GtkWidget* dialog =  gtk_message_dialog_new(
		                         GTK_WINDOW(m_window),
                                 GTK_DIALOG_DESTROY_WITH_PARENT,
                                 GTK_MESSAGE_ERROR,
                                 GTK_BUTTONS_CLOSE, str);
	gtk_dialog_run (GTK_DIALOG(dialog));
}
